use dotenv::dotenv;
use ldap3::{LdapConn, Mod};
use skynet_ldap_server::{get_config, Config};
use std::env;

#[async_std::main]
async fn main() -> tide::Result<()> {
    let config = get_config();

    //update_users(&config).await;
    update_admin(&config).await?;

    Ok(())
}

//async fn update_users(config: &Config) {

//}
fn uid_to_dn(uid: &str) -> String {
    format!("uid={},ou=users,dc=skynet,dc=ie", uid)
}

async fn update_admin(config: &Config) -> tide::Result<()> {
    dotenv().ok();

    // read from teh env
    if let Ok(x) = env::var("USERS_ADMIN") {
        let users = x.split(',').collect::<Vec<&str>>();

        update_group(config, "skynet-admins", &users, true).await?;
        // admins automatically get added as users
        update_group(config, "skynet-users", &users, false).await?;
    }
    Ok(())
}

async fn update_group(config: &Config, group: &str, users: &[&str], replace: bool) -> tide::Result<()> {
    let mut ldap = LdapConn::new(&config.ldap_host)?;

    // use the admin account
    ldap.simple_bind(&config.ldap_admin, &config.ldap_admin_pw)?.success()?;

    let dn = format!("cn={},ou=groups,dc=skynet,dc=ie", group);
    let members = users.iter().map(|uid| uid_to_dn(uid)).collect();
    let mods = if replace {
        vec![Mod::Replace("member".to_string(), members)]
    } else {
        vec![Mod::Add("member".to_string(), members)]
    };

    if let Err(x) = ldap.modify(&dn, mods) {
        println!("{:?}", x);
    }

    let dn_linux = format!("cn={}-linux,ou=groups,dc=skynet,dc=ie", group);
    let members_linux = users.iter().map(|uid| uid.to_string()).collect();
    let mods = if replace {
        vec![Mod::Replace("memberUid".to_string(), members_linux)]
    } else {
        vec![Mod::Add("memberUid".to_string(), members_linux)]
    };
    if let Err(x) = ldap.modify(&dn_linux, mods) {
        println!("{:?}", x);
    };

    // tidy up
    ldap.unbind()?;

    Ok(())
}
